/***********************************************************************
*
* Copyright (c) 2012-2016 Barbara Geller
* Copyright (c) 2012-2016 Ansel Sermersheim
* Copyright (c) 2012-2014 Digia Plc and/or its subsidiary(-ies).
* Copyright (c) 2008-2012 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
*
* This file is part of CopperSpice.
*
* CopperSpice is free software: you can redistribute it and/or 
* modify it under the terms of the GNU Lesser General Public License
* version 2.1 as published by the Free Software Foundation.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with CopperSpice.  If not, see 
* <http://www.gnu.org/licenses/>.
*
***********************************************************************/

#ifndef QSHAREDMEMORY_H
#define QSHAREDMEMORY_H

#include <QtCore/qobject.h>
#include <QScopedPointer>

QT_BEGIN_NAMESPACE

#ifndef QT_NO_SHAREDMEMORY

class QSharedMemoryPrivate;

class Q_CORE_EXPORT QSharedMemory : public QObject
{
   CORE_CS_OBJECT(QSharedMemory)
   Q_DECLARE_PRIVATE(QSharedMemory)

 public:
   enum AccessMode {
      ReadOnly,
      ReadWrite
   };

   enum SharedMemoryError {
      NoError,
      PermissionDenied,
      InvalidSize,
      KeyError,
      AlreadyExists,
      NotFound,
      LockError,
      OutOfResources,
      UnknownError
   };

   QSharedMemory(QObject *parent = 0);
   QSharedMemory(const QString &key, QObject *parent = 0);
   ~QSharedMemory();

   void setKey(const QString &key);
   QString key() const;
   void setNativeKey(const QString &key);
   QString nativeKey() const;

   bool create(int size, AccessMode mode = ReadWrite);
   int size() const;

   bool attach(AccessMode mode = ReadWrite);
   bool isAttached() const;
   bool detach();

   void *data();
   const void *constData() const;
   const void *data() const;

#ifndef QT_NO_SYSTEMSEMAPHORE
   bool lock();
   bool unlock();
#endif

   SharedMemoryError error() const;
   QString errorString() const;

 protected:
   QScopedPointer<QSharedMemoryPrivate> d_ptr;

 private:
   Q_DISABLE_COPY(QSharedMemory)

};

#endif // QT_NO_SHAREDMEMORY

QT_END_NAMESPACE

#endif // QSHAREDMEMORY_H

