/***********************************************************************
*
* Copyright (c) 2012-2016 Barbara Geller
* Copyright (c) 2012-2016 Ansel Sermersheim
* Copyright (c) 2012-2014 Digia Plc and/or its subsidiary(-ies).
* Copyright (c) 2008-2012 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
*
* This file is part of CopperSpice.
*
* CopperSpice is free software: you can redistribute it and/or 
* modify it under the terms of the GNU Lesser General Public License
* version 2.1 as published by the Free Software Foundation.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with CopperSpice.  If not, see 
* <http://www.gnu.org/licenses/>.
*
***********************************************************************/

#ifndef QGTKSTYLE_H
#define QGTKSTYLE_H

#include <QtGui/QCleanlooksStyle>
#include <QtGui/QPalette>
#include <QtGui/QFont>
#include <QtGui/QFileDialog>

QT_BEGIN_NAMESPACE

#if !defined(QT_NO_STYLE_GTK)

class QPainterPath;
class QGtkStylePrivate;

class Q_GUI_EXPORT QGtkStyle : public QCleanlooksStyle
{
   GUI_CS_OBJECT(QGtkStyle)
   Q_DECLARE_PRIVATE(QGtkStyle)

 public:
   QGtkStyle();
   QGtkStyle(QGtkStylePrivate &dd);

   ~QGtkStyle();

   QPalette standardPalette() const;

   void drawPrimitive(PrimitiveElement element, const QStyleOption *option, QPainter *painter, const QWidget *widget) const;
   void drawControl(ControlElement control, const QStyleOption *option, QPainter *painter, const QWidget *widget) const;
   void drawComplexControl(ComplexControl control, const QStyleOptionComplex *option, QPainter *painter, const QWidget *widget) const;
   void drawItemPixmap(QPainter *painter, const QRect &rect, int alignment, const QPixmap &pixmap) const;
   void drawItemText(QPainter *painter, const QRect &rect, int alignment, const QPalette &pal, bool enabled, const QString &text, 
         QPalette::ColorRole textRole) const;

   int pixelMetric(PixelMetric metric, const QStyleOption *option = 0, const QWidget *widget = 0) const;
   int styleHint(StyleHint hint, const QStyleOption *option, const QWidget *widget, QStyleHintReturn *returnData) const;

   QStyle::SubControl hitTestComplexControl(ComplexControl cc, const QStyleOptionComplex *opt,
         const QPoint &pt, const QWidget *w) const;

   QRect subControlRect(ComplexControl control, const QStyleOptionComplex *option, SubControl subControl, const QWidget *widget) const;
   QRect subElementRect(SubElement sr, const QStyleOption *opt, const QWidget *w) const;
   QRect itemPixmapRect(const QRect &r, int flags, const QPixmap &pixmap) const;

   QSize sizeFromContents(ContentsType type, const QStyleOption *option, const QSize &size, const QWidget *widget) const;
   QPixmap standardPixmap(StandardPixmap sp, const QStyleOption *option, const QWidget *widget) const;
   QPixmap generatedIconPixmap(QIcon::Mode iconMode, const QPixmap &pixmap, const QStyleOption *opt) const;

   void polish(QWidget *widget);
   void polish(QApplication *app);
   void polish(QPalette &palette);

   void unpolish(QWidget *widget);
   void unpolish(QApplication *app);

   static bool getGConfBool(const QString &key, bool fallback = 0);
   static QString getGConfString(const QString &key, const QString &fallback = QString());

 protected :
   QIcon standardIconImplementation(StandardPixmap standardIcon, const QStyleOption *option, const QWidget *widget = 0) const;

};

#endif //!defined(QT_NO_STYLE_QGTK)

QT_END_NAMESPACE

#endif //QGTKSTYLE_H
