/***********************************************************************
*
* Copyright (c) 2012-2016 Barbara Geller
* Copyright (c) 2012-2016 Ansel Sermersheim
* Copyright (c) 2012-2014 Digia Plc and/or its subsidiary(-ies).
* Copyright (c) 2008-2012 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
*
* This file is part of CopperSpice.
*
* CopperSpice is free software: you can redistribute it and/or 
* modify it under the terms of the GNU Lesser General Public License
* version 2.1 as published by the Free Software Foundation.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with CopperSpice.  If not, see 
* <http://www.gnu.org/licenses/>.
*
***********************************************************************/

#ifndef QSSLKEY_H
#define QSSLKEY_H

#include <QtCore/qnamespace.h>
#include <QtCore/qbytearray.h>
#include <QtCore/qsharedpointer.h>
#include <QtNetwork/qssl.h>

QT_BEGIN_NAMESPACE

#ifndef QT_NO_OPENSSL

template <typename A, typename B> struct QPair;

class QIODevice;
class QSslKeyPrivate;

class Q_NETWORK_EXPORT QSslKey
{

 public:
   QSslKey();
   QSslKey(const QByteArray &encoded, QSsl::KeyAlgorithm algorithm, QSsl::EncodingFormat format = QSsl::Pem, 
           QSsl::KeyType type = QSsl::PrivateKey, const QByteArray &passPhrase = QByteArray());

   QSslKey(QIODevice *device, QSsl::KeyAlgorithm algorithm, QSsl::EncodingFormat format = QSsl::Pem,
           QSsl::KeyType type = QSsl::PrivateKey, const QByteArray &passPhrase = QByteArray());

   QSslKey(const QSslKey &other);
   ~QSslKey();
   QSslKey &operator=(const QSslKey &other);

   bool isNull() const;
   void clear();

   int length() const;
   QSsl::KeyType type() const;
   QSsl::KeyAlgorithm algorithm() const;

   QByteArray toPem(const QByteArray &passPhrase = QByteArray()) const;
   QByteArray toDer(const QByteArray &passPhrase = QByteArray()) const;

   Qt::HANDLE handle() const;

   bool operator==(const QSslKey &key) const;
   inline bool operator!=(const QSslKey &key) const {
      return !operator==(key);
   }

 private:
   QExplicitlySharedDataPointer<QSslKeyPrivate> d;
   friend class QSslCertificate;
};

class QDebug;
Q_NETWORK_EXPORT QDebug operator<<(QDebug debug, const QSslKey &key);

#endif // QT_NO_OPENSSL

QT_END_NAMESPACE

#endif
