/***********************************************************************
*
* Copyright (c) 2012-2016 Barbara Geller
* Copyright (c) 2012-2016 Ansel Sermersheim
* Copyright (c) 2012-2014 Digia Plc and/or its subsidiary(-ies).
* Copyright (c) 2008-2012 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
*
* This file is part of CopperSpice.
*
* CopperSpice is free software: you can redistribute it and/or 
* modify it under the terms of the GNU Lesser General Public License
* version 2.1 as published by the Free Software Foundation.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with CopperSpice.  If not, see 
* <http://www.gnu.org/licenses/>.
*
***********************************************************************/

#ifndef QGLCOLORMAP_H
#define QGLCOLORMAP_H

#include <QtGui/qcolor.h>
#include <QtCore/qvector.h>

QT_BEGIN_NAMESPACE

class Q_OPENGL_EXPORT QGLColormap
{
 public:
   QGLColormap();
   QGLColormap(const QGLColormap &);
   ~QGLColormap();

   QGLColormap &operator=(const QGLColormap &);

   bool   isEmpty() const;
   int    size() const;
   void   detach();

   void   setEntries(int count, const QRgb *colors, int base = 0);
   void   setEntry(int idx, QRgb color);
   void   setEntry(int idx, const QColor &color);
   QRgb   entryRgb(int idx) const;
   QColor entryColor(int idx) const;
   int    find(QRgb color) const;
   int    findNearest(QRgb color) const;

 protected:
   Qt::HANDLE handle() {
      return d ? d->cmapHandle : 0;
   }
   void setHandle(Qt::HANDLE ahandle) {
      d->cmapHandle = ahandle;
   }

 private:
   struct QGLColormapData {
      QAtomicInt ref;
      QVector<QRgb> *cells;
      Qt::HANDLE cmapHandle;
   };

   QGLColormapData *d;
   static struct QGLColormapData shared_null;
   static void cleanup(QGLColormapData *x);
   void detach_helper();

   friend class QGLWidget;
   friend class QGLWidgetPrivate;
};

inline void QGLColormap::detach()
{
   if (d->ref.load() != 1) {
      detach_helper();
   }
}

QT_END_NAMESPACE

#endif // QGLCOLORMAP_H
