/***********************************************************************
*
* Copyright (c) 2012-2016 Barbara Geller
* Copyright (c) 2012-2016 Ansel Sermersheim
* Copyright (c) 2012-2014 Digia Plc and/or its subsidiary(-ies).
* Copyright (c) 2008-2012 Nokia Corporation and/or its subsidiary(-ies).
* All rights reserved.
*
* This file is part of CopperSpice.
*
* CopperSpice is free software: you can redistribute it and/or 
* modify it under the terms of the GNU Lesser General Public License
* version 2.1 as published by the Free Software Foundation.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with CopperSpice.  If not, see 
* <http://www.gnu.org/licenses/>.
*
***********************************************************************/

#ifndef QSQL_SQLITE_H
#define QSQL_SQLITE_H

#include <QtSql/qsqldriver.h>
#include <QtSql/qsqlresult.h>
#include <qsqlcachedresult_p.h>

struct sqlite3;

#ifdef QT_PLUGIN
#define Q_EXPORT_SQLDRIVER_SQLITE
#else
#define Q_EXPORT_SQLDRIVER_SQLITE Q_SQL_EXPORT
#endif

QT_BEGIN_NAMESPACE

class QSQLiteDriverPrivate;
class QSQLiteResultPrivate;
class QSQLiteDriver;

class QSQLiteResult : public QSqlCachedResult
{
   friend class QSQLiteDriver;
   friend class QSQLiteResultPrivate;

 public:
   explicit QSQLiteResult(const QSQLiteDriver *db);
   ~QSQLiteResult();
   QVariant handle() const;

 protected:
   bool gotoNext(QSqlCachedResult::ValueCache &row, int idx);
   bool reset(const QString &query);
   bool prepare(const QString &query);
   bool exec();
   int size();
   int numRowsAffected();
   QVariant lastInsertId() const;
   QSqlRecord record() const;
   void virtual_hook(int id, void *data);

 private:
   QSQLiteResultPrivate *d;
};

class Q_EXPORT_SQLDRIVER_SQLITE QSQLiteDriver : public QSqlDriver
{
   CS_OBJECT(QSQLiteDriver)
   friend class QSQLiteResult;

 public:
   explicit QSQLiteDriver(QObject *parent = 0);
   explicit QSQLiteDriver(sqlite3 *connection, QObject *parent = 0);
   ~QSQLiteDriver();
   bool hasFeature(DriverFeature f) const;
   bool open(const QString &db,
             const QString &user,
             const QString &password,
             const QString &host,
             int port,
             const QString &connOpts);
   void close();
   QSqlResult *createResult() const;
   bool beginTransaction();
   bool commitTransaction();
   bool rollbackTransaction();
   QStringList tables(QSql::TableType) const;

   QSqlRecord record(const QString &tablename) const;
   QSqlIndex primaryIndex(const QString &table) const;
   QVariant handle() const;
   QString escapeIdentifier(const QString &identifier, IdentifierType) const;

 private:
   QSQLiteDriverPrivate *d;
};

QT_END_NAMESPACE

#endif // QSQL_SQLITE_H
